@@ -9,12 +9,16 @@ use JSON;
 use LWP::UserAgent;
 use URI;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 $VERSION = eval $VERSION;
 
 sub new {
     my ($class, %params) = @_;
 
+    # These will be required at some point in the future.
+    # croak qq('appid' and 'token' are required);
+    #    unless ($params{appid} and $params{token}) {
+
     my $self = bless \ %params, $class;
 
     $self->ua(
@@ -25,8 +29,9 @@ sub new {
         my $dump_sub = sub { $_[0]->dump(maxlength => 0); return };
         $self->ua->set_my_handler(request_send  => $dump_sub);
         $self->ua->set_my_handler(response_done => $dump_sub);
+        $self->{compress} ||= 0;
     }
-    elsif (exists $self->{compress} ? $self->{compress} : 1) {
+    if (exists $self->{compress} ? $self->{compress} : 1) {
         $self->ua->default_header(accept_encoding => 'gzip,deflate');
     }
 
@@ -48,10 +53,10 @@ sub ua {
 sub geocode {
     my ($self, @params) = @_;
     my %params = (@params % 2) ? (location => @params) : @params;
+    my $raw = delete $params{raw};
+
+    $_ = Encode::encode('utf-8', $_) for values %params;
 
-    while (my ($key, $val) = each %params) {
-        $params{$key} = Encode::encode('utf-8', $val);
-    }
     my $location = delete $params{location} or return;
 
     if (my $language = delete $params{language}) {
@@ -60,26 +65,26 @@ sub geocode {
 
     my $uri = URI->new('http://where.desktop.mos.svc.ovi.com/NOSe/json');
     $uri->query_form(
-        q  => $location,
-        vi => 'where',
-        dv => 'oviMaps',
+        app_id => $self->{appid},
+        token  => $self->{token},
+        q      => $location,
+        vi     => 'where',
+        dv     => 'oviMapsAPI',
+        lat    => 0,
+        lon    => 0,
         %params,
     );
 
-    my $res = $self->{response} = $self->ua->get(
-        $uri, referer => 'http://maps.ovi.com/'
-    );
+    my $res = $self->{response} = $self->ua->get($uri);
     return unless $res->is_success;
 
     # Change the content type of the response from 'application/json' so
     # HTTP::Message will decode the character encoding.
     $res->content_type('text/plain');
 
-    my $content = $res->decoded_content;
-    return unless $content;
-
-    my $data = eval { from_json($content) };
+    my $data = eval { from_json($res->decoded_content) };
     return unless $data;
+    return $data if $raw;
 
     my @results = @{ $data->{results} || [] };
     return wantarray ? @results : $results[0];
@@ -98,9 +103,12 @@ Geo::Coder::Ovi - Geocode addresses with the Ovi Maps API
 
     use Geo::Coder::Ovi;
 
-    my $geocoder = Geo::Coder::Ovi->new;
+    my $geocoder = Geo::Coder::Ovi->new(
+        appid => 'Your App ID',
+        token => 'Your token',
+    );
     my $location = $geocoder->geocode(
-        location => 'Hollywood and Highland, Los Angeles, CA'
+        location => '102 Corporate Park Dr, Harrison, NY'
     );
 
 =head1 DESCRIPTION
@@ -112,27 +120,73 @@ of the Ovi Maps API.
 
 =head2 new
 
-    $geocoder = Geo::Coder::Ovi->new();
+    $geocoder = Geo::Coder::Ovi->new(
+        appid => 'Your App ID',
+        token => 'Your token',
+        # debug => 1,
+    )
 
 Creates a new geocoding object.
 
-Accepts an optional B<ua> parameter for passing in a custom LWP::UserAgent
-object.
+Accepts the following named arguments:
+
+=over
+
+=item * I<appid>
+
+=item * I<token>
+
+Authentication credentials. (optional, for now)
+
+Credentials can be obtained here: L<https://api.developer.nokia.com/ovi-api>
+
+Note: performance and/or access may be limited without credentials.
+
+=item * I<compress>
+
+Enable compression. (default: 1, unless I<debug> is enabled)
+
+=item * I<debug>
+
+Enable debugging. This prints the headers and content for requests and
+responses. (default: 0)
+
+=item * I<ua>
+
+A custom LWP::UserAgent object. (optional)
+
+=back
 
 =head2 geocode
 
     $location = $geocoder->geocode(location => $location)
     @locations = $geocoder->geocode(location => $location)
 
-Accepts an optional B<language> parameter to specify the preferred language
-of the response. The language code is composed of the two-letter ISO-639
-abbreviation and an optional two-letter ISO-3166 country code. Example
-values: 'en', 'en-US'.
+Accepts the following named arguments:
+
+=over
+
+=item * I<location>
+
+The free-form, single line address to be located. (required)
+
+=item * I<language>
+
+The preferred language of the response. The language may be specified as the
+ISO639-1 language code (e.g. C<en>) or the language code and the ISO3166-1
+alpha-2 country code (e.g. C<en-US>). (default: '')
+
+=item * I<raw>
+
+Returns the raw data structure converted from the response, not split into
+location results. (optional)
+
+=back
 
 In scalar context, this method returns the first location result; and in
 list context it returns all location results.
 
-Each location result is a hashref; a typical example looks like:
+Example of the data structure representing a location result:
 
     {
         categories => [ { id => 9000284 } ],
@@ -157,6 +211,14 @@ Each location result is a hashref; a typical example looks like:
         },
     }
 
+Example of the data structure returned using the I<raw> option:
+
+    {
+        id      => "mbc04bl15:20110629151643842:0000049",
+        results => [ $location ],
+        version => "1.0",
+    }
+
 =head2 response
 
     $response = $geocoder->response()
